
#include "Comparator.hxx"

/**
*	Define Comparator functions.
*/

namespace cqosdb {

	PropertySetComparator::PropertySetComparator(PropertySet& LHS, PropertySet& RHS, map<string, double>& tMap, map<string, int>& rMap){ 
	// compare if LHS is within {tolerance} of RHS
		setLHS(LHS);
		setRHS(RHS);
		toleranceMap = tMap;
		relationMap = rMap;	
	}
	
	PropertySetComparator::PropertySetComparator(PropertySet& LHS, PropertySet& RHS, map<string, int>& rMap){
		setLHS(LHS);
		setRHS(RHS);
		relationMap = rMap;
	}
	
	
	PropertySetComparator::PropertySetComparator(PropertySet& LHS, PropertySet& RHS){ 
	// compare if LHS is within {tolerance} of RHS
		setLHS(LHS);
		setRHS(RHS);
	}

	string PropertySetComparator::getClassName(){
		return string("PropertySetComparator");
	}
	
	PropertySetComparator::~PropertySetComparator(){
		clear();
	}
	
	void PropertySetComparator::clear(){
		toleranceMap.clear();
		relationMap.clear();
	}
	
	void PropertySetComparator::setLHS(ParameterSet& lefthand){ 
		string typeName("PropertySet");
		if (lefthand.getClassName().compare(typeName)!=0){
			fprintf(stderr, "Wrong left hand side type.\n");
			fflush(stderr);
			return;
		}

		lhs = dynamic_cast<PropertySet&>(lefthand);
	}
	
	void PropertySetComparator::setRHS(ParameterSet& righthand){ 
		string typeName("PropertySet");
		if (righthand.getClassName().compare(typeName)!=0){
			fprintf(stderr, "Wrong right hand side type.\n");
			fflush(stderr);
			return;
		}
		
		rhs = dynamic_cast<PropertySet&>(righthand);	
	}
			
	ParameterSet& PropertySetComparator::getLHS(){
		return lhs;
	}
	
	ParameterSet& PropertySetComparator::getRHS(){
		return rhs;
	}
	
	Parameter* PropertySetComparator::getLHSParameterAt(string paraName){
		return lhs.getParameterAt(paraName);
	}
	
	Parameter* PropertySetComparator::getRHSParameterAt(string paraName){
		return rhs.getParameterAt(paraName);
	}
			
	int PropertySetComparator::getDimension(){
		return lhs.getNumOfParameters();
	}
			
	void PropertySetComparator::setToleranceAt(string name, double epsilon){
			if (toleranceMap.find(name) != toleranceMap.end()){
				fprintf(stderr,"ERROR: tolerance for the property has already set!\n");
				fflush(stderr);
				return;
			}
			
			toleranceMap[name] = epsilon;
	}
	
	double PropertySetComparator::getToleranceAt(string name){
		if (toleranceMap.find(name) == toleranceMap.end()){
				fprintf(stderr,"ERROR: no tolerance set for the property!\n");
				fflush(stderr);
				return -1;
		}
		
		return toleranceMap[name];
	}
			
					
	void PropertySetComparator::setRelationAt(string name, int aRelation){
		if (relationMap.find(name) != relationMap.end()){
				fprintf(stderr,"ERROR: relation for the property has already set!\n");
				fflush(stderr);
				return;
		}
			
		relationMap[name] = aRelation;
	}
	
	int PropertySetComparator::getRelationAt(string name){
		if (relationMap.find(name) == relationMap.end()){
				fprintf(stderr,"ERROR: no relation set for the property!\n");
				fflush(stderr);
				return -1;
		}
		
		return relationMap[name];
	}
		
	bool PropertySetComparator::doCompare(){
		double aTolerance;
		int aRelation;
			
		if (!toleranceMap.empty()){
			map<string, double>::iterator iter;
			for (iter = toleranceMap.begin(); iter != toleranceMap.end(); iter++){
				// TODO : do the dynamic casts separately and check whether successful
				Parameter* leftM = dynamic_cast<Parameter*>(getLHSParameterAt(iter->first));
				Parameter* rightM = dynamic_cast<Parameter*>(getRHSParameterAt(iter->first));
				aTolerance = iter->second;
				aRelation = relationMap[iter->first];
				
				if (!leftM->compareWith(*rightM, aTolerance, aRelation))
					return false;
			}
			
			return true;
		}
		else{
			map<string, int>::iterator iter;
			for (iter = relationMap.begin(); iter != relationMap.end(); iter++){
				// TODO : do the dynamic casts separately and check whether successful
				Parameter* leftM = dynamic_cast<Parameter*>(getLHSParameterAt(iter->first));
				Parameter* rightM = dynamic_cast<Parameter*>(getRHSParameterAt(iter->first));
				aRelation = iter->second;
				
				if (!leftM->compareWith(*rightM, aRelation))
					return false;
			}
			
			return true;
		}
	}
			
   /***********  end of PropertySetComparator *************************/
}
