/**
 * \file Utility.hxx implements a set of generic operations.
 * \author Li Li
 * \date 4/30/08
 */

#ifndef UTILITY_HXX_
#define UTILITY_HXX_

#include <sstream>
#include <fstream>
#include <map>

using namespace std;

namespace cqosdb {
	
	/**
		A class intended to perform some generic operations.
	*/
class Utility {
	
public:

	/** 
		Convert from a T to a string Type T must support << operator
	*/
	template<class T> static string T_as_string( const T& t );
	
	/**
		Convert from a string to a T Type T must support >> operator
	*/
	template<class T> static T string_as_T( const string& s );
	
	/**
		Remove leading whitespace
	*/
	static void trim( string& s );
	
	/**
        Convert a string to lower case.
	*/
	static void stringToLower(char *str,int len);

};


template<class T>
string Utility::T_as_string( const T& t )
{
	ostringstream ost;
	ost << t;
	return ost.str();
}


template<class T>
T Utility::string_as_T( const string& s )
{
	T t;
	istringstream ist(s);
	ist >> t;
	return t;
}


/**
	Convert from a string to a string
*/
template<>
inline string Utility::string_as_T<string>( const string& s )
{
	return s;
}


/**
	Convert from a string to a bool. Interpret "false", "F", "no", "n", "0" as false; 
	Interpret "true", "T", "yes", "y", "1", "-1", or anything else as true
*/
template<>
inline bool Utility::string_as_T<bool>( const string& s )
{
	
	bool b = true;
	string sup = s;
	for( string::iterator p = sup.begin(); p != sup.end(); ++p )
		*p = toupper(*p);  // make string all caps
	if( sup==string("FALSE") || sup==string("F") ||
	    sup==string("NO") || sup==string("N") ||
	    sup==string("0") || sup==string("NONE") )
		b = false;
	return b;
}

}

#endif


